package com.neuri.webfixture;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.thoughtworks.selenium.DefaultSelenium;
import com.thoughtworks.selenium.Selenium;
import com.thoughtworks.selenium.Wait;

import fitlibrary.SequenceFixture;

public class PlainSeleniumTest extends SequenceFixture {
	
	/** Pattern to find the label part of the options list. */
	private static final String FORMULA_PATTERN = "[^(label *= *)]";
	
	/** Compiled pattern to find the label part of the options list. */
	private static final Pattern COMPILED_FORMULA_PATTERN = Pattern.compile(FORMULA_PATTERN, Pattern.CASE_INSENSITIVE);
	
	protected Selenium instance;
	public void getComDotNeuriDotWebfixtureDotPlainSeleniumTest(){
		// required by fitnesse 20080812
	}
	protected String getBrowserCode(String browser){
		 if ("IE".equalsIgnoreCase(browser)) return "*iehta";
		 if ("FIREFOX".equalsIgnoreCase(browser)) return "*chrome";
		 return browser;
	}
	protected String baseUrl;
     public void startBrowser
         (String browser, String rcServer, int rcPort, String seleniumURL)
     {
    	 baseUrl=seleniumURL;
         instance = new DefaultSelenium(rcServer, rcPort, getBrowserCode(browser), seleniumURL);
         instance.start();
         this.setSystemUnderTest(instance);
     }
     public void shutdownBrowser()
     {
         instance.stop();
     }
     // override open to allow for relative URLs
	public void open (String url){
		if (!url.startsWith("http://"))
			instance.open(baseUrl+url);
		else
			instance.open(url);
	}
	// general timeout for waiters
	public int timeout=10;
	public void setTimeout(int timeout){
		this.timeout=timeout;
	}

    /**************************************************/
	/* waiter subclasses */
	protected class WaitForTextToAppear extends Wait{
		protected String text;
	   	 public  WaitForTextToAppear(String  text){
	   		this.text= text; 
	   	 }
	   	 public boolean until(){
	   		 return instance.isTextPresent(text);
	   	}
    }
	protected class WaitForChecked extends Wait{
		protected String locator;
		protected boolean checked;
	   	 public  WaitForChecked (String  locator, boolean checked){
	   		this.locator=locator;
	   		this.checked=checked;
	   	 }
	   	 public boolean until(){
	   		 if (!instance.isElementPresent(locator)) return false;
	   		 return instance.isChecked(locator)==checked;
	   	}
   }
	protected class WaitForElementToAppear extends Wait{
		protected String text;
	   	 public  WaitForElementToAppear(String  text){
	   		this.text= text; 
	   	 }
	   	 public boolean until(){
	   		 return instance.isElementPresent(text);
	   	}
   }
	protected class WaitForElementToDisappear extends Wait{
		protected String text;
	   	 public  WaitForElementToDisappear(String  text){
	   		this.text= text; 
	   	 }
	   	 public boolean until(){
	   		 return (!instance.isElementPresent(text));
	   	}
   }
    
	protected class WaitForFieldValue extends Wait{
    	protected String text;
      	protected String elementLocator;
      	 public  WaitForFieldValue(String element, String  value){   		 
      		 this.text= value; 
      		 this.elementLocator=element;
      	 }
      	 public boolean until(){    		 
          	 String value=instance.getValue(elementLocator);
          	 if (value==null) return false;
      		 return value.indexOf(text)>=0;
   		}
       }
    /**************************************************/
	
	// utility method for pausing
	public void pause(int milliseconds){
   	 try{
   		 Thread.sleep(milliseconds);
   	 }
   	 catch (InterruptedException iex){
   		 //nothing important, just silently ignore
   	 }
    }
	public void pause(int milliseconds,String ignore){
		pause(milliseconds);
	}

	public void open (String url, String ignore){
		open(url);
	}
	public void click(String what, String ignore){
		instance.click(what);
	}
	public void clickAndWait(String what, String milliseconds){
        instance.click(what);
        instance.waitForPageToLoad(milliseconds);
    }
	public void click(String what){
		instance.click(what);
	}
    public void clickAndWait(String what){
        instance.click(what);
        instance.waitForPageToLoad(Integer.toString(timeout*1000));
    }
	public boolean waitForTextPresent(String text, String ignore){
		return waitForTextPresent(text);
	}
    public boolean waitForTextPresent(String text){
            Wait x=new WaitForTextToAppear(text);
            x.wait("Cannot find text " +text+ " after "+timeout+" seconds",timeout*1000);
            return true;    	
    }
    public boolean waitForElementPresent(String text, String ignore){
		return waitForElementPresent(text);
    }
    public boolean waitForElementPresent(String locator){
        Wait x=new WaitForElementToAppear(locator);
        x.wait("Cannot find element " +locator+ " after "+timeout+" seconds",timeout*1000);
        return true;    	
    }
    public boolean waitForElementNotPresent(String text, String ignore){
		return waitForElementNotPresent(text);
    }
    public boolean waitForElementNotPresent(String locator){
        Wait x=new WaitForElementToDisappear(locator);
        x.wait("Element " +locator+ " found after "+timeout+" seconds",timeout*1000);
        return true;    	
    }
    public boolean waitForFieldValue(String field, String value){
    	  Wait x=new WaitForFieldValue(field, value);
    	  x.wait("Cannot find value" +value+ " in " + field +" after "+timeout+" seconds",timeout*1000);
          return true;
    }
    ////////// verify methods
    public boolean verifyElementPresent(String locator){
    	return verifyElementPresent(locator,true);
    }
    public boolean verifyElementPresent(String locator, boolean present){
    	return instance.isElementPresent(locator)==present;
    }
    public boolean verifyTextPresent(String locator){
    	return verifyTextPresent(locator,true);
    }
    public boolean verifyTextPresent(String locator, boolean present){
    	return instance.isTextPresent(locator)==present;
    }
    public boolean verifyTextNotPresent(String locator){
    	return verifyTextNotPresent(locator,true);
    }
    public boolean verifyTextNotPresent(String locator, boolean notpresent){
    	return instance.isTextPresent(locator)!=notpresent;
    }
    public boolean verifySelectedLabel(String locator, String label){
    	String label1=instance.getSelectedLabel(locator);
    	System.out.println("Selected label for "+locator+"="+label1);
    	return label1.equals(label);
    }
    public boolean verifyValue(String locator, String value){
    	String label1=instance.getValue(locator);
    	System.out.println("Value of  "+locator+"="+label1);
    	return label1.equals(value);
    }
    public boolean verifyNotValue(String locator, String value){
    	return !verifyValue(locator, value);
    }
    public boolean verifyText(String locator, String value){
    	String label1=instance.getText(locator);
    	System.out.println("text in "+locator+"="+label1);
    	return label1.equals(value);
    }
    public boolean verifyVisible(String locator){
    	return verifyVisible(locator,true);
    }
    public boolean verifyVisible(String locator, boolean visible){
    	return instance.isVisible(locator)==visible;
    }
    public boolean verifySelectOptions(String locator, String values) {
    	return verifySelectOptions(locator, values, true);
    }
    public boolean verifySelectOptions(String locator, String values, boolean present) {
    	String[] options=instance.getSelectOptions(locator);
    	final List<String> optionList = createSortedList(removeLabel(options));
    	final List<String> valueList = createSortedList(removeLabel(values.split(",")));
    	
    	return optionList.equals(valueList) == present;
    }
    /**
     * Remove the labels from the texts.
     * @param textsWithLabel The texts.
     * @return The texts without the labels.
     */
    protected String[] removeLabel(String[] textsWithLabel) {
    	String[] textsWithoutLabel = new String[textsWithLabel.length];
    	List<String> list = new ArrayList<String>(textsWithLabel.length);
    	for (String textWithLabel: textsWithLabel) {
    		list.add(removeLabel(textWithLabel));
    	}
    	return list.toArray(textsWithoutLabel);
    }
    /**
     * Remove the label (e.g: 'label =', 'LABEL=' etc.) from the text.  
     * @param textWithLabel The text with a possible label.
     * @return The text without the label.
     */
    protected String removeLabel(String textWithLabel) {
    	String textWithoutLabel = textWithLabel;
        Matcher m = COMPILED_FORMULA_PATTERN.matcher(textWithLabel);
        if (m.find()) {
        	textWithoutLabel = textWithLabel.substring(m.start());
        }

    	return textWithoutLabel;
    }
    public boolean verifyNotSelectOptions(String locator, String values) {
    	return !verifySelectOptions(locator, values);
    }
    public boolean verifyNotSelectOptions(String locator, String values, boolean present) {
    	return !verifySelectOptions(locator, values, present);
    }
	
	/**
	 * Convert an array of strings to a sorted {@linkplain List}.
	 * @param values The values to convert to a list.
	 * @return The sorted list.
	 * @see http://www.exampledepot.com/egs/java.util/coll_FindInList.html
	 */
    protected List<String> createSortedList(String[] values) {
	    List<String> sortedList = new LinkedList<String>();
	    for (String value: values) {
	    	insert(sortedList, value.trim());
	    }
	    
	    return sortedList;
	}
	/**
	 * Insert an item in a sorted {@linkplain List}. 
	 * @param sortedList The sorted list.
	 * @param item The item to insert.
	 * @see http://www.exampledepot.com/egs/java.util/coll_FindInList.html
	 */
	protected void insert(final List<String> sortedList, String item) {
	    // Search for the non-existent item
	    int index = Collections.binarySearch(sortedList, item);
	    
	    // Add the non-existent item to the list
	    if (index < 0) {
	        sortedList.add(-index-1, item);
	    }
	}
	
    public boolean verifyElementNotPresent(String locator){
    	return !verifyElementPresent(locator);
    }
    public boolean verifyElementNotPresent(String locator, boolean notpresent){
    	return instance.isElementPresent(locator)!=notpresent;
    }
    public boolean verifyEditable(String locator){
    	return verifyEditable(locator,true);
    }
    public boolean verifyEditable(String locator, boolean editable){
    	return instance.isEditable(locator)==editable;
    }
    public boolean verifyNotEditable(String locator){
    	return !verifyEditable(locator);
    }
    public boolean verifyNotEditable(String locator, boolean noteditable){
    	return instance.isEditable(locator)!=noteditable;
    }
    public boolean verifyChecked(String locator){
    	return verifyChecked(locator,true);
    }
    public boolean verifyChecked(String locator, boolean checked){
    	return instance.isChecked(locator)==checked;
    }
    public boolean verifyNotChecked(String locator){
    	return !verifyChecked(locator);
    }
    public boolean verifyNotChecked(String locator, boolean notchecked){
    	return instance.isChecked(locator)!=notchecked;
    }
    public boolean waitForChecked(String locator,boolean checked){
        Wait x=new WaitForChecked(locator,checked);
        x.wait("Checkbox " +locator+ " is "+(checked?"not":"still") +" checked after "+timeout+" seconds",timeout*1000);
        return true;
    }
    public boolean waitForChecked(String locator){
    	return waitForChecked(locator,true);
    }    
    // map to allow typing in an empty string
    public void type(String locator){
    	instance.type(locator,"");
    }
    ///// alerts and confirmations
    public boolean assertAlertPresent(String alert){
    	return alert.equals(instance.getAlert());
    }
    public boolean assertConfirmation(String s){
    	return s.equals(instance.getConfirmation());
    }
    ///// select options 
    public boolean assertSelectOptions(String locator, String options){
    	return options.equals(getSelectOptions(locator));
    }
    public String getSelectOptions(String locator){
    	StringBuilder sb=new StringBuilder();
    	boolean notfirst=false;
    	for (String s:instance.getSelectOptions(locator)){
    		if (notfirst)
    			sb.append(", ");
    		sb.append(s);
    		notfirst=true;
    	}
    	return sb.toString();
    }
    
    /**
     * Get an instance of the {@linkplain Selenium} object.
     * @return The Selenium object.
     */
    Selenium getInstance() {
    	return instance;
    }
}
